#!/usr/bin/python


def getStatus(V , Wn , Cr=0 , Mn=0 , Mx=0) :
	Cr = int(Cr)
	Wn = int(Wn)
	V = int(V)

	if Cr != 0 :
		if V < Wn and Wn < Cr  : 
			return 0
		elif Wn <= V and V < Cr : 
			return 1
		elif Wn <= Cr and Cr <= V  :
			return 2
		else :
			return 3
	elif Cr == 0 :
                if V < Wn  :
                        return 0
                elif V >= Wn  :
                        return 1
                else :
                        return 3



#!/usr/bin/python 

def getDescText () :
        return  "Options to be passed as checks  \
                backends  -->> Checks for the number of clients/connections currently connected to the specified database\
                connections  -->> Tests the databases if they currently accept connections or not  \
                wals  -->> keeps tab on the number/size of wal files present in the pg_xlog directory  \
                vacuum -->> Inspects the tables and produces a report of those that has not been vauumed for a given period of time  \
                autovacuum -->> Same as vacuum except that this is triggered by the database's internal processes  \
                analyze  -->> reports tables that have not been analyzed after a given period of time  \
                autoanalyze  -->> Same as analyze but triggered by the database's internal processes. \
                table_bloat  -->> Inspect all tables and reports the bloat percentage which is usually a function of vacuum and/or autovacuuming not running regularly  \
                index_bloat -->> Same as table bloat, except for indexes. This could be remedied by reindexing  \
                blocking  -->> This checks for locks transactional/relational that are blocking other processes from running.  \
                nonblocking -->> This checks for locks that are running more than the specified type. This normally do not block, but could become a blocking lock \
                if it runs longer than necessary. In some cases, it is the long running locks that become the blocking types  \
                table_size -->> Monitors table sizes  \
                index_size -->> Monitors index sizes  \
                database_size  -->> Monitors database sizes  \
                checkpoints -->> Monitors checkpoint frequency  \
                duplicate_indexes -->> Monitors indexes and report duplicates  \
                replica_lag -->> Monitors all replicas connected to a given master "



#!/usr/bin/python 

# list of valid actions 
# backends 
#connections
# wals
# autovacuum
# vacuum
# autoanalyze 
# analyze 
# tale_bloat 
# index_bloat 
# table_size
# index_size 
# database_size 
# nonblocking
# blocking
# checkpoints 
# duplicate_indexes
# replica_lag

import factors as fac


def getDefaults (check , warning , critical ) :
	if check == 'backends' : 
		# only whole digits and percentages allowed, an or separator is also allowed 
		return fac.getNumberPercentMix (warning, critical, '80%', '85%' )
	elif check == 'connections' :
                return {'warning':'dummy', 'critical' :'dummy'}
	elif check == 'autovacuum' or check == 'vacuum' or check == 'autoanalyze' or check == 'analyze':
		crit = critical if critical != None else '0min'
		warn = warning if warning != None else '1month'
		if fac.checkDigit(crit.split()) == True and fac.checkDigit(warn.split()) == True :
			return {'warning' : warn , 'critical' : crit}
		else :
			return None		
	elif check == 'table_size' or check == 'index_size' or check == 'database_size' :
		crit = critical if critical != None else '0kb'
		warn = warning
		if warn == None and crit != '0kb' :
			warn = critical
		elif warn == None and crit == '0kb' :
			return None

		if fac.checkDigit(crit.split()) == True and fac.checkDigit(warn.split()) == True :
                        return {'warning' : warn , 'critical' : crit}
                else :
                        return None
	elif check == 'table_bloat' or check == 'index_bloat' :
		crit = critical if critical != None else '0kb'
		warn = warning if warning != None else '1gb'
		if fac.checkDigit(crit.split()) == True and fac.checkDigit(warn.split()) == True :
			return {'warning' : warn , 'critical' : crit}
		else :
                        return None
	elif check == 'nonblocking' or check == 'blocking' :
                crit = critical if critical != None else '0min'
                warn = warning if warning != None else '1min'
                if fac.checkDigit(crit.split()) == True and fac.checkDigit(warn.split()) == True :
                        return {'warning' : warn , 'critical' : crit}
                else :
                        return None  
	elif check == 'replica_lag' :
		return fac.getNumberPercentMix (warning, critical, '5', '10' ) 
	else :
		if warning != None or critical != None :
			warn = warning 
			crit = critical if critical != None else 0			


                        if crit != 0 and warning == None :
                        	warn = warning if warning != None else crit
			

			if check == 'wals' :
				# -- warning and/or critical must be supplied
				if str(warn).isdigit() and str(crit).isdigit()  :
					return {'warning' : warn , 'critical' : crit}
				else :
					return None  
			#elif check == 'checkpoints' :
				# warning and/or critical value must be supplied
			#	return None
		else :
			print ("Warning and/or critical value required for this check")
			return None 



#!/usr/bin/python

import sys
import checkStatus as st
import sql
import factors as fac
import hexa
import perfdataText as perf

# var=val;warn;crit;min;max
# check_mk output format 
# status 	item_name	perfdata	output


def getReplicaLags( param=None ) :
	item_name = 'POSTGRES_REPLICA_LAG'
	status = []
	perfdata = '-'
	output = ''
	if param != None :
		
		warning = int( param['warning'] )
		critical = int( param['critical'] )

                query = "SELECT pg_current_xlog_location() "
		
		dbname = param.get('dbname')[0]
		
		hosts = param['host']
		ports = param['port']	
		if len(hosts) > len(ports) :
			ports = ports * len(hosts)

		master = sql.getSQLResult ( {'host': hosts[0] , 'port' : ports[0], 'dbname': dbname, 'user' : param['user'] ,'password' : param['password'] } ,query ) 
		
		
		if master[0] == None :
			return '2' + ' ' + item_name  + ' ' + '-' + ' ' + str(master[1])

		counter = 1
		for rep_host in hosts[1:] :
			query = "SELECT pg_last_xlog_replay_location()"
			replica = sql.getSQLResult ( {'host': rep_host , 'port' : ports[counter], 'dbname': dbname \
					, 'user' : param['user'] ,'password' : param['password'] } ,query )
			if replica[0] == 0 :
				wal_lag = hexa.computeMegaByteDiff ( master[1] , replica[1] ) / 16
			elif replica[0] == None :
				wal_lag = -1 
			if perfdata == '-' :
				perfdata = perf.getPerfStm (rep_host , wal_lag ,warning,str(critical)) 
				output =  'replica on {0:s} lags behind master {1:s} by a total of {2:s} WALs. NOTE: a -1 lag value means the replica is not available !!! '\
					  .format( rep_host , hosts[0] , str(wal_lag) )
			elif perfdata != '-'  :
				perfdata = perfdata + '|' + perf.getPerfStm (rep_host , wal_lag ,warning,str(critical))
				output =  output + ';\n replica on {0:s} lags behind master {1:s} by a total of {2:s} WALs. NOTE: a -1 lag value means the replica is not available !!!'\
						.format( rep_host , hosts[0] , str(wal_lag) )
			if wal_lag != -1 :
				status.append( st.getStatus( wal_lag , warning , critical ) )
			elif wal_lag == -1 :
				status.append(2)
			counter += 1

		status.sort(reverse=True)
		return str(status[0]) + ' ' + item_name + ' ' + str(perfdata) + ' ' + output



#!/usr/bin/python

import math
import re

def getTimeFactor ( check_val ) :
	check_val = str(check_val)
        factor = 0
        val = re.findall(r'\d+', check_val.lower() )[0]
        unit = ''
        if check_val.lower().find('min') != -1  or check_val.lower().find('minute') != -1:
                factor = 1
                unit = 'min'
        elif check_val.lower().find('hr') != -1 or check_val.lower().find('hour') != -1 :
                factor = 60
                unit = 'hr'
        elif check_val.lower().find('day') != -1 or check_val.lower().find('dy') != -1 :
                factor = 24*60
                unit = 'day'
        elif check_val.lower().find('wk') != -1 or check_val.lower().find('week') != -1 :
                factor = 60 * 24 * 7
                unit = 'week'
	elif check_val.lower().find('month') != -1 or check_val.lower().find('mon') != -1 :
                factor = 60 * 24 * 7 * 4
                unit = 'month'	
        elif check_val.lower().find('yr')  != -1 or check_val.lower().find('year')  != -1 :
                factor = 60 * 24 * 7 * 52
                unit = 'year'

	if str(val).isdigit() :
        	return [val , factor, unit]
	else :
		return None


def getSizeFactor ( check_val ) :
	check_val = str(check_val)
        factor = 0
        val = re.findall(r'\d+', check_val.lower() )[0]
        unit = ''
        if check_val.lower().find('kb') != -1 :
                factor = 1024
                unit = 'KB'
        elif check_val.lower().find('mb') != -1 :
                factor = 1024 * 1024
                unit = 'MB'
        elif check_val.lower().find('gb') != -1  :
                factor = 1024 * 1024 * 1024
                unit = 'GB'
        elif check_val.lower().find('tb') != -1 :
                factor = 1024 * 1024 * 1024 * 1024
                unit = 'TB'
        elif check_val.lower().find('pb') != -1  :
                factor = 1024 * 1024 * 1024 * 1024 * 1024
                unit = 'PB'

	if str(val).isdigit() :
                return [val , factor, unit]
        else :
                return None


def getNumberPercentLimits( limit, total ) :
        if limit != None :
                pLimit = str(limit).split('or')
                nperc = []
		
                for ele in pLimit :
			val = re.findall(r'\d+',ele)[0]
			val = math.ceil( ( float ( val ) / 100 ) * total )  if ele.find('%') != -1 else int ( val ) 
			nperc.append (val) 
                return int (max( nperc))



def checkDigit (check_val) :
	retval = []
	for e in check_val :
		val = re.findall(r'\d+', e )[0] if len(re.findall(r'\d+', e ) ) == 0 else 'text'			
        	retval.append( val )
        return False not in  [True for element in retval if element.isdigit()]

	

def getNumberPercentMix (warning=None, critical=None, defaultWarn=None, defaultCritical=None ) :
        retval = {}
        warn = warning   if warning != None else defaultWarn
        crit = critical  if critical != None else defaultCritical
        retval.update ({'warning' : warn})
        retval.update ({'critical' : crit})

        if checkDigit(str(warn).split('or'))  and  checkDigit(str(crit).split('or'))  : 
                return retval
        else :
                return None


def warningAndOrCriticalProvided (warning,critical) :
        # as an heuristic 
        # warning = 0.8 * critical
	if warning != None and critical != None :
		warning = getSizeFactor ( warning )
		critical = getSizeFactor ( critical )
		retval = {}
		if ( int(critical[0]) == 0 ) or  ( ( int(critical[0]) != 0 )  and ( int(critical[0]) > int(warning[0])  ) ) :
			retval.update( {'warning' : warning }  )
			retval.update( {'critical' : critical }  )
		else :
			retval = None

		return retval
	else :
		return None




def getTimeDefaults ( warning, critical , defaultWarning=None , defaultCritical=None ) :
                warn = warning if warning != None else defaultWarning
                crit = critical if critical != None else defaultCritical
                warn = getTimeFactor( warn )
                crit = getTimeFactor( crit )
                retval = {}
                if warn != None and crit != None :
			# check for the larger of the 2
			if ( int(crit[0]) == 0 ) or ( int(crit[0]) != 0 )  and ( int(crit[0]) > int(warn[0])  ) :
                        	retval.update( {'warning' : warn }  )
                        	retval.update( {'critical' : crit }  )
			else :
				retval = None 
                        return retval
                else :
                        return None




#!/usr/bin/python

import sql 


def getBackends( param=None ) :
        item_name = 'POSTGRES_DUPLICATE_INDEXES'
        status = []
        perfdata = '-'
        output = ''
        if param != None :

		dbnames = param.get('dbname')[0]
                query ="WITH index_cols_ord as ( \
    				SELECT attrelid, attnum, attname \
    				FROM pg_attribute \
        			JOIN pg_index ON indexrelid = attrelid \
    				WHERE indkey[0] > 0 \
    				ORDER BY attrelid, attnum \
			), \
				index_col_list AS ( \
    				SELECT attrelid, \
        			array_agg(attname) as cols \
    				FROM index_cols_ord \
    				GROUP BY attrelid \
			), \
			dup_natts AS ( \
				SELECT indrelid, indexrelid \
				FROM pg_index as ind \
				WHERE EXISTS ( SELECT 1 \
    				FROM pg_index as ind2 \
    				WHERE ind.indrelid = ind2.indrelid \
    				AND ( ind.indkey @> ind2.indkey \
     				OR ind.indkey <@ ind2.indkey ) \
    				AND ind.indkey[0] = ind2.indkey[0] \
    				AND ind.indkey <> ind2.indkey \
    				AND ind.indexrelid <> ind2.indexrelid \
			) ) \
			SELECT \
				userdex.schemaname  || '.' || userdex.relname || '.' || userdex.indexrelname as relname,\
				array_to_string(cols, ', ') as index_cols, \
				indexdef, \
				idx_scan as index_scans \
			FROM  \
				pg_stat_user_indexes as userdex \
    			JOIN index_col_list ON index_col_list.attrelid = userdex.indexrelid \
    			JOIN dup_natts ON userdex.indexrelid = dup_natts.indexrelid \
    			JOIN pg_indexes ON userdex.schemaname = pg_indexes.schemaname \
        			AND userdex.indexrelname = pg_indexes.indexname \
			ORDER BY userdex.schemaname, userdex.relname, cols, userdex.indexrelname LIMIT 5" 

                exclude_db = param.get('exclude_db')
                for db in exclude_db :
                        if db in dbnames :
                                dbnames.remove(db)

		for dbname in dbnames :
                	results = sql.getSQLResult ( {'host': param['host'][0] , 'port' : param['port'][0], 'dbname': dbname, 'user' : param['user'] ,'password' : param['password'] } ,query )
			if results[0] == None : 
				return '2' + ' ' + item_name + ' ' + perfdata  + ' ' + results[1]
		
			rows = results[1]

			if len(rows) > 0 :
                		for row in rows :
                        		if output == '' :
                                		output =  '{0:s} with cols {1:s} appears to be a duplicate Index'.format(row[0],row[1])
                        		elif output != ''  :
                                		output =  output + ';\n {0:s} with cols {1:s} appears to be a duplicate Index'.format(row[0],row[1])


		if output != '' :
			return '1' + ' ' + item_name + ' ' + '-' + ' ' + output
		else :
			return '0' + ' ' + item_name + ' ' + '-' + ' ' + 'OK'




#!/usr/bin/python 

import time
import sql
import math


def getConnections( param=None ) :
        item_name = 'POSTGRES_CONNECTIONS'
        status = []
        perfdata = '-'
        output = ''
        if param != None :
                query = "SELECT \
                              datname \
                         FROM \
                              pg_database \
                         WHERE datistemplate is FALSE"
	
		host = param['host'][0]
		port = param['port'][0]
		dbname = param.get('dbname')[0]

                results = sql.getSQLResult ( {'host': host , 'port' : port, 'dbname': dbname, 'user' : param['user'] ,'password' : param['password'] } ,query )
		if results[0] == None :
			return '2' + ' ' + 'POSTGRES_CONNECTIONS' + ' ' + '-' + ' ' + results[1]
	
		dbs = results[1]

                exclude_db = param.get('exclude_db')
                for db in exclude_db :
                        if db in dbs :
                                dbnames.remove(db)

		for db in dbs :
			query = "select version()"
			begin = time.time()
			row = sql.getSQLResult ( {'host': host , 'port' : port, 'dbname': db[0], 'user' : param['user'] ,'password' : param['password'] } ,query )
			duration = int ( math.ceil ( ( time.time() - begin ) * 1000 ) )
			out_add = 'connection test took {0:s} ms '
			if row != None :
				status.append(0)
			else :
				status.append(2)
				out_add = ' refuses connection !!! {0:s}'
				duration = ''

                       	if perfdata == '-' :
                               	perfdata = db[0] + '=' + str( duration )
                               	output =  '{0:s} {1:s}'.format(db[0], out_add.format(str(duration) ) )
                       	elif perfdata != '-'  :
                               	perfdata = perfdata + '|' + db[0] + '=' + str ( duration )
                               	output =  output + ';\n {0:s} {1:s}'.format(db[0], out_add.format(str(duration) ) )


                status.sort( reverse=True )
                return str(status[0]) + ' ' + item_name + ' ' + str(perfdata) + ' ' + output



#!/usr/bin/python

# find param : ipl_api.init_de_request
import defaults, helpText 
import monitor
import sys


def getHelpText() :
	print ( helpText.getDescText() ) 

def getArgs(param) :
	if param != None :	
		warning = param.get('warning')
		critical = param.get('critical')
		check_warning_critical = defaults.getDefaults (param.get('check'), warning, critical )

		if check_warning_critical != None :
			param['warning'] = check_warning_critical.get('warning')
			param['critical'] = check_warning_critical.get('critical') 
			print (monitor.getChecks (param))
		else : 
			getHelpText()
			sys.exit(0)



#!/usr/bin/python

import sys
import checkStatus as st
import sql
import factors as fac

# var=val;warn;crit;min;max
# check_mk output format 
# status 	item_name	perfdata	output


def getBackends( param=None ) :
	item_name ='POSTGRES_BACKENDS'
	status = []
	perfdata = '-'
	output = ''
	if param != None :  # extra safety check, but absolutely not necessary 

		dbnames = param.get('dbname')		
		exclude = param.get('exclude_db') if param.get('exclude_db') != None else None
		exclude_db = ''
		if exclude != None :
			for exc in exclude :
				exclude_db = " AND dbs.datname <> " + "'" + exc + "'"  	
		 
                query = "SELECT \
			      dbs.datname, \
                              ( current_setting('max_connections')::int - current_setting('superuser_reserved_connections')::int ) as max_allowed_connect,\
                              coalesce( nullif((numbackends -1 ),-1), 0 ) as connect_count,\
                              ( coalesce( nullif((numbackends -1 ),-1), 0 ) * 100 ) / current_setting('max_connections')::int as percent_connect \
                         FROM \
				pg_stat_database dbs \
                         JOIN \
				pg_database db on db.datname = dbs.datname \
                         WHERE \
				db.datistemplate IS FALSE {0:s} ".format(exclude_db)
				
		results = sql.getSQLResult ( {'host': param['host'][0] , 'port' : param['port'][0], 'dbname': dbnames[0], 'user' : param['user'] ,'password' : param['password'] } ,query ) 
		connect_sum = 0

		if results[0] == None :
			return '2' + ' ' + item_name  + ' ' + '-' + ' ' + str(results[1])

		rows = results[1]
		if len(rows) > 0  :
			max_connect = rows[0][1]
			warning = fac.getNumberPercentLimits(  param['warning'] , max_connect )
			critical = fac.getNumberPercentLimits(  param['critical'] , max_connect )
			for row in rows :
				if int(row[2]) != 0 :
					if perfdata == '-' :
						perfdata = row[0] + '=' + str(row[2]) + ';' +  str(warning) + ';' + str(critical) + ';' + '1' + ';' + str(row[1]) 
						output =  '{0:s} has {1:d}% of the total connections used'.format(row[0],row[3])
					elif perfdata != '-'  :
						perfdata = perfdata + '|' + row[0] + '=' + str(row[2]) + ';' +  str(warning) + ';' + str(critical) + ';' + '1' + ';' + str(row[1])
						output =  output + ';{0:s} has {1:d}% of the total connections used'.format(row[0],row[3])
					connect_sum += int(row[2])		
				else :
					if perfdata == '-' :
                                        	perfdata = row[0] + '=' + str(row[2]) + ';' +  str(warning) + ';' + str(critical) + ';' + '1' + ';' + str(row[1])
                                        	output =  'No backend currenlty exists for {0:s} database - which might implies that the Application connected to the database might be down or the database currently refuses connections to it '.format(row[0])
                                	elif perfdata != '-'  :
                                        	perfdata = perfdata + '|' + row[0] + '=' + str(row[2]) + ';' +  str(warning) + ';' + str(critical) + ';' + '1' + ';' + str(row[1])
                                        	output =  output + ';No backend currenlty exists for {0:s} database - which might implies that the Application connected to the database might be down or the database currently refuses connections to it '.format(row[0])
					status.append(2)
			status.append( st.getStatus( int(connect_sum),int(warning) , int(critical), int('0') , int(max_connect)  ) )
			
			status.sort( reverse=True )
			return str(status[0]) + ' ' + item_name + ' ' + str(perfdata) + ' ' + output





#!/usr/bin/python

import sql 
import factors as fac
import perfdataText as perf
import checkStatus as st


def getReturnIndex(check) :
	if check.lower() == 'vacuum' :
		return [1,"pg_stat_user_tables.last_vacuum","pg_stat_user_tables.vacuum_count"]
	elif check.lower() == 'autovacuum' :
		return [1,"pg_stat_user_tables.last_autovacuum","pg_stat_user_tables.autovacuum_count" ]
	elif check.lower() == 'analyze' :
		return [1,"pg_stat_user_tables.last_analyze", "pg_stat_user_tables.analyze_count" ] 
	elif check.lower() == 'autoanalyze' :
		return [1,"pg_stat_user_tables.last_autoanalyze", "pg_stat_user_tables.autoanalyze_count"]


def getVacuums( param=None ) :
        item_name = 'POSTGRES_'
        status = []
        perfdata = '-'
        output = ''
	warning = []
	critical = []
        if param != None :
	
		dbnames = param.get('dbname')
		check = param['check']
		item_name = item_name + check.upper()
		index =  getReturnIndex(check)
		retval = fac.getTimeDefaults ( param.get('warning'), param.get('critical') )
		if retval != None :
			warning = retval.get('warning') 
			critical = retval.get('critical') 
		else  :
			return '2' + ' ' + item_name  + ' ' + '-' + ' ' + 'Invalid Parameters supplied !'	 

		query = "SELECT table_name,calculated_time FROM ( \
				SELECT (schemaname || '.' || relname) table_name,\
				CASE WHEN {0:s} IS NULL THEN  \
					date_part('epoch',clock_timestamp() - '1970-01-01 00:00:00'::timestamp ) /60  \
				ELSE \
					date_part('epoch',clock_timestamp() - {0:s} ) /60 \
				END AS calculated_time \
			FROM \
				pg_stat_user_tables \
				) foo  \
		 	WHERE \
				( calculated_time /60 )  >=  ( {1:d} * {2:d} ) \
                                ORDER BY calculated_time DESC LIMIT 5 ".format( index[1] , int(warning[0]) , int(warning[1]) )

                exclude_db = param.get('exclude_db')
                for db in exclude_db :
                        if db in dbnames :
                                dbnames.remove(db)

		for dbname in dbnames :
                	results = sql.getSQLResult ( {'host': param['host'][0] , 'port' : param['port'][0], 'dbname': dbname, 'user' : param['user'] ,'password' : param['password'] } ,query )
		
			if results[0] == None : 
				return '2' + ' ' + item_name  + ' ' + perfdata + ' ' + results[1]
	
			rows = results[1]


			if len(rows) > 0 :
                		for row in rows : 
					status.append( st.getStatus( int(row[index[0]]) , int(warning[0]) * int(warning[1]) , int(critical[0]) * int(critical[1]) )  ) 
					out_unit = str ( int(row[1]/(60*24)) ) + ' days ago ' 
                        		if perfdata == '-' :
                                		perfdata = perf.getPerfStm (row[0],str(row[index[0]]),str(warning[0]),str(critical[0]))
                                		output =  '{0:s} last {1:s} time was {2:s} '.format(row[0],check,out_unit)
                        		elif perfdata != '-'  :
                                		perfdata = perfdata + '|' + perf.getPerfStm (row[0],str(row[index[0]]),str(warning[0]),str(critical[0])) 
                                		output =  output + ';\n {0:s} last {1:s} time was {2:s}'.format(row[0],check,out_unit)
	        		
				status.sort( reverse=True )


		if perfdata != '-' :
                	return str(status[0]) + ' ' + item_name + ' ' + str(perfdata) + ' ' + output 
		else :
			return str('0') + ' ' + item_name + ' ' + '-'  + ' ' + 'OK'



#!/usr/bin/python

import psycopg2 as pg
import psycopg2.errorcodes


def getSQLResult ( connParam  ,query ) :
	host = connParam['host']
	user = connParam['user']
	port = connParam['port']
	dbname = connParam['dbname']
	password = connParam['password']
	row = None
	
        try :
		conn_str = "host={0:s} user={1:s} port={2:s} dbname={3:s} password={4:s}".format( host , user , str(port) , dbname , password )
                conn = pg.connect( conn_str )
                cur = conn.cursor()
                cur.execute(query)
                row = cur.fetchall()
		conn.close()
		cur.close()
		return [ 0 , row] 
        except pg.Error as err :
		return [None , str(err) ]
		



#!/usr/bin/python

import sql
import factors as fac
import checkStatus as st
import perfdataText as perf

def getVersionParam (version) :
        query = ''
        pid = ''
        if version >= 9.2 :
                query = 'query'
                pid = 'pid'
        elif version <= 9.1 :
                query = 'current_query'
                pid = 'procpid'

	return [query, pid]

def getNonBlockingVersionQuery (version,w1,w2) :
	query , pid = getVersionParam (version) 
	return "SELECT \
			l.relation::regclass::text as table_name ,\
			l.locktype,\
			pa.{0:s},\
			pa.{1:s}, \
			pa.usename ,\
			pa.client_addr,\
                        date_part('epoch',clock_timestamp()::timestamp - pa.query_start::timestamp)/60 running_time, \
			l.mode \
                FROM \
			 pg_catalog.pg_locks l \
                join pg_catalog.pg_stat_user_tables ut on ut.schemaname || '.' || ut.relname = l.relation::regclass::text \
                join pg_catalog.pg_database db on db.oid = l.database \
                join pg_catalog.pg_stat_activity pa on  l.pid = pa.{0:s} \
                where l.granted and lower(l.mode) = any ( \
                         values(lower('ExclusiveLock')),(lower('AccessExclusiveLock')), (lower('ShareLock')) , (lower('RowExclusiveLock')) , \
                         (lower('RowShareLock'))  ) \
                         And date_part('epoch',clock_timestamp()::timestamp - pa.query_start::timestamp)/60 >= ( {2:d} * {3:d} ) \
			LIMIT 5 ".format(pid,query, int(w1), int(w2) ) 

def getBlockingVersionQuery (version) :
	query , pid = getVersionParam (version)
	return "SELECT \
			a.{0:s}	 	AS blocked_statement, \
			ka.{0:s}	AS blocking_statement,\
                        kl.locktype     AS blocking_type, \
                        bl.locktype     AS blocked_type, \
                        ka.usename      AS blocking_user, \
                        kl.pid		AS blocking_pid, \
                        date_part('epoch',clock_timestamp()::timestamp - ka.query_start::timestamp)/60  blocking_time, \
                        bl.pid          AS blocked_pid, \
                        a.usename       AS blocked_user, \
                        date_part('epoch',clock_timestamp()::timestamp - a.query_start::timestamp)/60  waiting_time \
  		FROM  \
			pg_catalog.pg_locks         bl \
   		JOIN pg_catalog.pg_stat_activity a  ON a.{1:s} = bl.pid \
   		JOIN pg_catalog.pg_locks         kl ON kl.transactionid = bl.transactionid AND kl.pid != bl.pid \
   		JOIN pg_catalog.pg_stat_activity ka ON ka.{1:s} = kl.pid \
  		WHERE NOT bl.GRANTED LIMIT 5".format( query , pid)

def getBlockingIterator(rows,item_name,findText, status,warning,critical) :
        perfdata = '-'
        output = ''
        for row in rows :
                if perfdata == '-' :
                        perfdata = perf.getPerfStm ('Blocking_query',row[6],warning[0],str(critical[0]))
                        output =  '{0:s}(pid->{1:s}) has been blocked({2:s}) by {3:s}(pid->{4:s}) for {5:s} mins \n  Blocked Query : {6:s} \
                                   \n  Blocking Query : {7:s} \n Waiting time : {8:s}'.format( str(row[8]) , str(row[7]) , str(row[2]) , str(row[4]), \
                                    str(row[5]), str(row[6]), str(row[0]), str(row[1]), str(row[9])  )
                elif perfdata != '-'  :
                        perfdata = perfdata + '|' + perf.getPerfStm ('Blocking_query',row[6],1,'0')
			output =  output + '; \n {0:s}(pid->{1:s}) has been blocked({2:s}) by {3:s}(pid->{4:s}) for {5:s} mins \n  Blocked Query : {6:s} \
                                   \n  Blocking Query : {7:s} \n Waiting time : {8:s}'.format( str(row[8]) , str(row[7]) , str(row[2]) , str(row[4]), \
                                    str(row[5]), str(row[6]), str(row[0]), str(row[1]), str(row[9])  )


	return {'status' : '2', 'item' : item_name , 'perfdata' : perfdata, 'output' : output }


def getNonBlockingIterator(rows,item_name,warning,critical, status) :
	perfdata = '-'
	output = ''
	for row in rows :
		if perfdata == '-' :
                	perfdata = perf.getPerfStm (row[0],row[6],warning[0],str(critical[0]))
                        output =  '{0:s} has been locked({1:s}) by {2:s}(pid->{3:s},lock_mode->{6:s}) for {4:s} mins \n Locking Query : {5:s}'.format( \
				  str(row[0]),str(row[1]),str(row[4]),str(row[2]),str(row[6]), str(row[3]), str(row[7])  )
                elif perfdata != '-'  :
                	perfdata = perfdata + '|' + perf.getPerfStm (row[0],row[6],warning[0],str(critical[0]))
                        output =  output + ';\n {0:s} has been locked({1:s}) by {2:s}(pid->{3:s},lock_mode->{6:s}) for {4:s} mins \n Locking Query : {5:s}'.format( \
                                  str(row[0]),str(row[1]),str(row[4]),str(row[2]),str(row[6]), str(row[3]),str(row[7])  )
		status.append( st.getStatus( row[6],int(warning[0]) , int(critical[0])  ) )
	status.sort( reverse=True )
	return {'status' : status[0], 'item' : item_name , 'perfdata' : perfdata, 'output' : output }


def getLocks( param=None ) :

        item_name = 'POSTGRES_'
        status = []
        perfdata = '-'
        output = ''
        if param != None :
		check = (param['check']).lower()
		findText = param.get('find')
		item_name = item_name + check.upper() + '_LOCKS'
		dbnames = param.get('dbname')
		user = param['user']
		password = param.get('password')
		host = param['host'][0]
		port = param['port'][0]
		warning = []
		critical = []

		query = "SELECT substring(version() FROM '(\d.\d)')::double precision"
                results = sql.getSQLResult ( {'host': host , 'port' : port, 'dbname': dbnames[0], 'user' : param['user'] ,'password' : param['password'] } ,query )
		
		if results[0] == None :
                        return '2' + ' ' + item_name + ' ' + '-' + ' ' + results[1]
		

		version = (results[1])[0][0]
		retvals = fac.getTimeDefaults ( param.get('warning'), param.get('critical') ) 
		if retvals !=None :
			warning = retvals.get('warning')
			critical = retvals.get('critical')
		else :
			return '2' + ' ' + item_name + ' ' + '-' + ' ' + 'Invalid parameter passed !'
		
		results = []
		exclude_db = param.get('exclude_db')
		for db in exclude_db :
			if db in dbnames :
				dbnames.remove(db)

		for dbname in dbnames :
			if check == 'nonblocking' :
				results = sql.getSQLResult ( {'host': host , 'port' : port , 'dbname': dbname,\
					 'user' : user ,'password' : password } ,getNonBlockingVersionQuery(version,warning[0], warning[1])  )
			elif check == 'blocking' :
				results  = sql.getSQLResult ( {'host': host , 'port' : port , 'dbname': dbname,\
                                	 'user' : user ,'password' : password } ,getBlockingVersionQuery(version)  )

			if results[0] == None :
				return '2' + ' ' + item_name + ' ' + perfdata + ' ' + results[1]

			retval = {}
			if len(results[1]) > 0 and check == 'nonblocking' : 	
				retval = getNonBlockingIterator(results[1],'POSTGRES_NONBLOCKING_LOCKS',warning,critical, status)
			elif len(results[1]) > 0 and check == 'blocking' :
				retval = getBlockingIterator(results[1],'POSTGRES_BLOCKING_LOCKS',findText, status,warning,critical)

			status.append(retval.get('status'))
			if perfdata != '-' and len(results[1]) > 0 :
				perfdata = perfdata + retval.get('perfdata')
				output = retval.get('output')
			elif perfdata == '-' and len(results[1]) > 0  :
				perfdata = retval.get('perfdata')
				output = output + ';\n' + retval.get('output')	

		if perfdata != '-' :
			status.sort(reverse=True)
			return status[0] + ' ' + item_name + ' ' +  perfdata  + ' ' + output
		else : 
			return '0' + ' ' + item_name + ' ' + '-' + ' ' + 'OK'	




#!/usr/bin/python

def getPerfStm (check,val,warning,critical,Min=None,Max=None) :
        perf = ''
        if critical != '0' :
                perf = check + '=' + str(val) + ';' +  str(warning) + ';' + str(critical)   
        elif critical == '0' :
                perf = check + '=' + str(val) + ';' +  str(warning)   
        return perf




#!/usr/bin/python

import backends as bac
import bloats as blt
import connection as con
import duplicateIndexes as dup
import locks as loc
import relationSizes as rel
import vacuum as vac
import wals
import replicaLag as rep

def getChecks (param) :
	check = param['check']
        if check == 'backends' :
                return bac.getBackends (param)
        elif check == 'wals' :
                return wals.getWALs(param)
        elif check == 'autovacuum' or check == 'vacuum' or check == 'autoanalyze' or check == 'analyze':
		return vac.getVacuums(param)
        elif check == 'table_bloat' or check == 'index_bloat' :
		return blt.getBloats(param)
        elif check == 'table_size' or check == 'index_size' or check == 'database_size' :
		return rel.getRelationSizes(param)
        elif check == 'nonblocking' or check == 'blocking' :
		return loc.getLocks(param)
        elif check == 'checkpoints' :
		return None
        elif check == 'replica_lag' :
		return rep.getReplicaLags(param) 
	elif check == 'connections' :
		return con.getConnections(param)



#!/usr/bin/python

import sql 
import factors as fac
import checkStatus as st
import perfdataText as perf


def getQuery (check) :
	if check == 'table_bloat' :
		return "WITH constants AS ( \
    			SELECT current_setting('block_size')::numeric AS bs, 23 AS hdr, 8 AS ma \
			),\
			no_stats AS ( \
    			SELECT table_schema, table_name, \
        		n_live_tup::numeric as est_rows,\
        		pg_table_size(relid)::numeric as table_size \
    			FROM information_schema.columns \
        		JOIN pg_stat_user_tables as psut \
           		ON table_schema = psut.schemaname \
           		AND table_name = psut.relname \
        		LEFT OUTER JOIN pg_stats \
        		ON table_schema = pg_stats.schemaname \
            		AND table_name = pg_stats.tablename \
            		AND column_name = attname  \
    			WHERE attname IS NULL \
        		AND table_schema NOT IN ('pg_catalog', 'information_schema') \
    			GROUP BY table_schema, table_name, relid, n_live_tup \
		),\
		null_headers AS ( \
    			SELECT \
        		hdr+1+(sum(case when null_frac <> 0 THEN 1 else 0 END)/8) as nullhdr, \
        		SUM((1-null_frac)*avg_width) as datawidth, \
        		MAX(null_frac) as maxfracsum, \
        		schemaname,\
        		tablename,\
        		hdr, ma, bs \
    			FROM pg_stats CROSS JOIN constants \
        		LEFT OUTER JOIN no_stats \
            		ON schemaname = no_stats.table_schema \
            		AND tablename = no_stats.table_name \
    			WHERE schemaname NOT IN ('pg_catalog', 'information_schema') \
        		AND no_stats.table_name IS NULL \
        		AND EXISTS ( SELECT 1 \
            		FROM information_schema.columns \
                	WHERE schemaname = columns.table_schema \
                    	AND tablename = columns.table_name ) \
    			GROUP BY schemaname, tablename, hdr, ma, bs \
		),\
		data_headers AS ( \
    			SELECT \
        		ma, bs, hdr, schemaname, tablename, \
        		(datawidth+(hdr+ma-(case when hdr%ma=0 THEN ma ELSE hdr%ma END)))::numeric AS datahdr,\
        		(maxfracsum*(nullhdr+ma-(case when nullhdr%ma=0 THEN ma ELSE nullhdr%ma END))) AS nullhdr2 \
    			FROM null_headers \
		),\
		table_estimates AS ( \
    			SELECT schemaname, tablename, bs,\
        		reltuples::numeric as est_rows, relpages * bs as table_bytes,\
    			CEIL((reltuples*\
            		(datahdr + nullhdr2 + 4 + ma - (CASE WHEN datahdr%ma=0 THEN ma ELSE datahdr%ma END) \
                	)/(bs-20))) * bs AS expected_bytes, \
        		reltoastrelid \
    			FROM data_headers \
        		JOIN pg_class ON tablename = relname \
        		JOIN pg_namespace ON relnamespace = pg_namespace.oid \
            		AND schemaname = nspname \
   			WHERE pg_class.relkind = 'r' \
		),\
		estimates_with_toast AS ( \
    			SELECT schemaname, tablename,  \
        		TRUE as can_estimate, \
        		est_rows, \
        		table_bytes + ( coalesce(toast.relpages, 0) * bs ) as table_bytes, \
        		expected_bytes + ( ceil( coalesce(toast.reltuples, 0) / 4 ) * bs ) as expected_bytes \
    			FROM table_estimates LEFT OUTER JOIN pg_class as toast \
        		ON table_estimates.reltoastrelid = toast.oid \
            		AND toast.relkind = 't' \
		),\
		table_estimates_plus AS ( \
    			SELECT current_database() as databasename,\
            		schemaname, tablename, can_estimate, \
            		est_rows, \
            		CASE WHEN table_bytes > 0 \
                	THEN table_bytes::NUMERIC \
                	ELSE NULL::NUMERIC END \
                	AS table_bytes, \
            		CASE WHEN expected_bytes > 0  \
                	THEN expected_bytes::NUMERIC \
                	ELSE NULL::NUMERIC END \
                    	AS expected_bytes,\
            		CASE WHEN expected_bytes > 0 AND table_bytes > 0 \
                	AND expected_bytes <= table_bytes \
                	THEN (table_bytes - expected_bytes)::NUMERIC \
                	ELSE 0::NUMERIC END AS bloat_bytes \
    			FROM estimates_with_toast \
    			UNION ALL \
    			SELECT current_database() as databasename,  \
        		table_schema, table_name, FALSE, \
        		est_rows, table_size,\
        		NULL::NUMERIC, NULL::NUMERIC\
    			FROM no_stats\
		),\
		bloat_data AS ( \
    			select current_database() as databasename,\
        		schemaname, tablename, can_estimate, \
        		expected_bytes, expected_bytes as expected_mb,\
        		round(bloat_bytes*100/table_bytes) as pct_bloat,\
        		bloat_bytes ,table_bytes, expected_bytes, est_rows\
    			FROM table_estimates_plus\
		)\
		SELECT \
			schemaname || '.' || tablename AS table_name,\
			table_bytes AS table_size_bytes,\
			pct_bloat AS percentage_bloat,\
			round(bloat_bytes/{0:d}) AS bloat_size_bytes\
		FROM \
			bloat_data \
		WHERE \
			bloat_bytes/{0:d} >= {1:d}  \
		ORDER BY  \
			pct_bloat DESC LIMIT 5"


	elif check == 'index_bloat' :
		return "WITH btree_index_atts AS ( \
    			SELECT nspname, \
        		indexclass.relname as index_name, \
        		indexclass.reltuples, \
        		indexclass.relpages, \
        		indrelid, indexrelid,\
        		indexclass.relam,\
       	 		tableclass.relname as tablename,\
        		regexp_split_to_table(indkey::text, ' ')::smallint AS attnum,\
        		indexrelid as index_oid \
    			FROM pg_index \
    			JOIN pg_class AS indexclass ON pg_index.indexrelid = indexclass.oid \
    			JOIN pg_class AS tableclass ON pg_index.indrelid = tableclass.oid \
    			JOIN pg_namespace ON pg_namespace.oid = indexclass.relnamespace \
    			JOIN pg_am ON indexclass.relam = pg_am.oid \
    			WHERE pg_am.amname = 'btree' and indexclass.relpages > 0 \
         		AND nspname NOT IN ('pg_catalog','information_schema') \
    		),\
		index_item_sizes AS ( \
    			SELECT \
    			ind_atts.nspname, ind_atts.index_name,  \
    			ind_atts.reltuples, ind_atts.relpages, ind_atts.relam, \
    			indrelid AS table_oid, index_oid, \
    			current_setting('block_size')::numeric AS bs, \
    			8 AS maxalign, \
    			24 AS pagehdr, \
    			CASE WHEN max(coalesce(pg_stats.null_frac,0)) = 0 \
        		THEN 2 \
        		ELSE 6 \
    			END AS index_tuple_hdr, \
    			sum( (1-coalesce(pg_stats.null_frac, 0)) * coalesce(pg_stats.avg_width, 1024) ) AS nulldatawidth \
    			FROM pg_attribute \
    			JOIN btree_index_atts AS ind_atts ON pg_attribute.attrelid = ind_atts.indexrelid AND pg_attribute.attnum = ind_atts.attnum \
    			JOIN pg_stats ON pg_stats.schemaname = ind_atts.nspname \
          		AND ( (pg_stats.tablename = ind_atts.tablename AND pg_stats.attname = pg_catalog.pg_get_indexdef(pg_attribute.attrelid, pg_attribute.attnum, TRUE)) \
          		OR   (pg_stats.tablename = ind_atts.index_name AND pg_stats.attname = pg_attribute.attname)) \
    			WHERE pg_attribute.attnum > 0 \
    			GROUP BY 1, 2, 3, 4, 5, 6, 7, 8, 9 \
		), \
		index_aligned_est AS ( \
    			SELECT maxalign, bs, nspname, index_name, reltuples,\
        		relpages, relam, table_oid, index_oid, \
        		coalesce ( \
            		ceil ( \
                	reltuples * ( 6  \
                    	+ maxalign \
                    	- CASE \
                        WHEN index_tuple_hdr%maxalign = 0 THEN maxalign \
                        ELSE index_tuple_hdr%maxalign \
                      	END \
                    	+ nulldatawidth \
                    	+ maxalign \
                    	- CASE \
                        WHEN nulldatawidth::integer%maxalign = 0 THEN maxalign \
                        ELSE nulldatawidth::integer%maxalign \
                      	END \
                	)::numeric \
              		/ ( bs - pagehdr::NUMERIC ) \
              		+1 ) \
         		, 0 ) \
      			as expected \
    			FROM index_item_sizes \
		),\
		raw_bloat AS ( \
    			SELECT current_database() as dbname, nspname, pg_class.relname AS table_name, index_name, \
        		bs*(index_aligned_est.relpages)::bigint AS totalbytes, expected, \
        		CASE \
            		WHEN index_aligned_est.relpages <= expected \
                	THEN 0 \
                	ELSE bs*(index_aligned_est.relpages-expected)::bigint \
            		END AS wastedbytes, \
        		CASE \
            		WHEN index_aligned_est.relpages <= expected \
                	THEN 0 \
                	ELSE bs*(index_aligned_est.relpages-expected)::bigint * 100 / (bs*(index_aligned_est.relpages)::bigint) \
            		END AS realbloat, \
        		pg_relation_size(index_aligned_est.table_oid) AS table_bytes, \
        		stat.idx_scan AS index_scans \
    			FROM index_aligned_est \
    			JOIN pg_class ON pg_class.oid=index_aligned_est.table_oid \
    			JOIN pg_stat_user_indexes AS stat ON index_aligned_est.index_oid = stat.indexrelid \
		), \
		format_bloat AS ( \
			SELECT dbname AS database_name, nspname as schema_name, table_name, index_name, \
        		round(realbloat) as bloat_pct, wastedbytes AS bloat_bytes, \
        		totalbytes AS index_bytes, \
        		table_bytes, \
        		index_scans \
			FROM raw_bloat \
		) \
		SELECT \
			schema_name || '.' || '.' || table_name || '.' || index_name AS table_name, \
			table_bytes  AS table_size_bytes, \
			bloat_pct AS bloat_percentage, \
			round(bloat_bytes/{0:d})  AS bloat_size_bytes, \
			index_bytes  AS index_size_bytes, \
			index_scans \
		FROM format_bloat \
		WHERE bloat_bytes/{0:d} >= {1:d}   \
		ORDER BY bloat_bytes DESC LIMIT 5"


def getBloats( param=None ) :
        item_name = 'POSTGRES_'
        status = []
        perfdata = '-'
        output = ''
        if param != None :
		check = param['check']
		item_name = item_name + check.upper()
		dbnames = param.get('dbname')
		retval = fac.warningAndOrCriticalProvided (param.get('warning'),param.get('critical'))
		warning = []
		critical = []
		if retval != None :
			warning = retval.get('warning')
			critical = retval.get('critical')
		else :
			return '2' + ' ' + item_name + ' ' + '-' + ' ' + 'Invalid parameters supplied'
                query = getQuery(check)
		query = query.format(int(warning[1]),int(warning[0]))

                exclude_db = param.get('exclude_db')
                for db in exclude_db :
                        if db in dbnames :
                                dbnames.remove(db)

		
		for dbname in dbnames :
                	results = sql.getSQLResult ( {'host': param['host'][0] , 'port' : param['port'][0], 'dbname': dbname, 'user' : param['user'] ,'password' : param['password'] } ,query )

			if results[0] == None :
				return '2' + ' ' + item_name + ' ' + perfdata + ' ' + results[1]
			
			rows = results[1]
			if len(rows) > 0 :
                		for row in rows :
                                	out_unit = ''
					status.append(st.getStatus(row[3] , warning[0] , critical[0]) )

                        		if perfdata == '-' :
                                		perfdata = perf.getPerfStm (row[0],row[3],warning[0],str(critical[0]))
                                		output =  '{0:s} has {1:s} {2:s} ({3:s})% worth of bloat'.format(row[0],str(row[3]), warning[2], str(row[2]) )
                        		elif perfdata != '-'  :
                                		perfdata = perfdata + '|' + perf.getPerfStm (row[0],row[3],warning[0],str(critical[0]))
                                		output =  output + ';\n {0:s} has {1:s} {2:s} ({3:s})% worth of bloat'.format(row[0],str(row[3]), warning[2], str(row[2]) )

                	
		if perfdata != '-' :
			status.sort( reverse=True )
			return str(status[0]) + ' ' + item_name + ' ' + str(perfdata) + ' ' + output
		else : 
			return '0' + ' ' + item_name  + ' ' + '-' + ' ' + 'OK'



#!/usr/bin/python

import math

def getDecimalEquivalent(hexadigit) :
	hexaDict = {'0' : 0, '1' : 1, '2' : 2 , '3' : 3, '4' : 4, '5' : 5 , '6' : 6 , '7' : 7, '8' : 8 , '9' : 9, 'A' : 10, \
		   'B' : 11, 'C' : 12, 'D' : 13, 'E' : 14, 'F' : 15}

	return hexaDict.get( str(hexadigit) )


def convertHex2Dec (hexa) :
# example - C921
	dec = 0
	counter = 1
	for hexadigit in str(hexa).upper() :
		dec = dec + getDecimalEquivalent(hexadigit) * 16 ** ( len(hexa) - counter )
		counter += 1

	return dec

def computeMegaByteDiff (master , replica ) :
	multiplier = convertHex2Dec('FF000000')
	byte = ( multiplier * convertHex2Dec( str(master[0][0]).split('/')[0] )  + convertHex2Dec( str(master[0][0]).split('/')[1] ) )  - \
		( multiplier * convertHex2Dec( str(replica[0][0]).split('/')[0] )  + convertHex2Dec( str(replica[0][0]).split('/')[1] ) ) 
	return int( math.ceil(byte / (1024 ** 2)) )

#if __name__ == '__main__' :
#	print (convertHex2Dec ('D4445B8'))



#!/usr/bin/python 

from setuptools import setup

def readme():
    with open('README.txt') as f:
        return f.read()

setup(name='pg_monitor',
      version='1.6.2',
      description='PostgreSQL monitoring checks',
      long_description=readme(),
      classifiers=[
        'Development Status :: 5 - Production/Stable',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
        'Topic :: System :: Networking :: Monitoring',
      ],
      url='https://github.com/rugging24/pg_monitor',
      author='Olakunle Olaniyi',
      author_email='rugging24@gmail.com',
      license='PostgreSQL License',
      packages=['pg_monitor'],
      install_requires=[
          'psycopg2',
      ],
      scripts=['bin/pg_monitor'],
      include_package_data=True,
      zip_safe=False)



#!/usr/bin/python

import sql
import checkStatus as st
import factors as fac
import perfdataText as perf

def getQuery ( check,warn_factor,warn_val ) :
	query = ''
	col_name = check.replace('size','name')
	if check == 'table_size' or check == 'index_size':
		query = "SELECT \
			{0:s} , \
			pg_relation_size({0:s})/{1:d}  FROM (\
			SELECT relid::regclass table_name,indexrelid::regclass index_name \
			FROM pg_stat_user_indexes \
			) foo \
			WHERE pg_relation_size({0:s}) / {1:d}  >= {2:d}	".format(col_name,int(warn_factor),int(warn_val))
	elif check == 'database_size' :
		query = "SELECT \
	                      datname, \
			      (pg_database_size(datname) / {0:d} ) as display_size \
                         FROM \
	                      pg_database \
                         WHERE \
	                      datistemplate IS FALSE \
                         AND \
	                      ( pg_database_size(datname) / {0:d} ) >= {1:d}".format(int(warn_factor),int(warn_val))

	return query 

def getRelationSizes( param=None ) :
        item_name = 'POSTGRES_'
        status = []
        perfdata = '-'
        output = 'OK'
	warning = []
	critical = []
        if param != None :
		retval = fac.warningAndOrCriticalProvided (param.get('warning'),param.get('critical')) 
		if retval != None :
			warning = retval.get('warning')
			critical = retval.get('critical')
		else :
			return '2' + ' ' + item_name  + ' ' + '-' + ' ' + 'Invalid parameters passed !'

		item_name = item_name + str(param['check']).upper()
		col_name = 'table_name' 
                query = getQuery ( param['check'],warning[1],warning[0] ) 
		dbnames = param.get('dbname')

                exclude_db = param.get('exclude_db')
                for db in exclude_db :
                        if db in dbnames :
                                dbnames.remove(db)

		for dbname in dbnames : 
                	results = sql.getSQLResult ( {'host': param['host'][0] , 'port' : param['port'][0], 'dbname': dbname, 'user' : param['user'] ,'password' : param['password'] } ,query )
		
			if results[0] == None : 
				return '2' + ' ' + item_name  + ' ' + perfdata + ' ' + results[1]

			rows = results[1]

			if len(rows) > 0 :
                		for row in rows :
					status.append( st.getStatus(row[1] , warning[0] , critical[0] )  )
                        		if perfdata == '-' :
                                		perfdata = perf.getPerfStm (row[0],row[1],warning[0],critical[0])
                                		output =  '{0:s} is {1:d} {2:s} big'.format(row[0],row[1],warning[2])
                        		elif perfdata != '-'  :
                                		perfdata = perfdata + '|' + perf.getPerfStm (row[0],row[1],warning[0],critical[0])
                                		output =  output + ';\n {0:s} is {1:d} {2:s} big'.format(row[0],row[1],warning[2])

		if perfdata != '-' :
			status.sort( reverse=True )
			return str(status[0]) + ' ' + item_name + ' ' + str(perfdata) + ' ' + output
		else :
                	return '0' + ' ' + item_name + ' ' + str(perfdata) + ' ' + output





#!/usr/bin/python

import sql 
import checkStatus as st
import factors as fac
import perfdataText as perf


def getWALs( param=None ) :
        item_name = 'POSTGRES_WALS'
        status = []
        perfdata = '-'
        output = ''
        if param != None :
		dbname = param.get('dbname')[0]
                query = "SELECT count(bar.*) wal_count , \
				ceil(( ( 2 + current_setting('checkpoint_completion_target')::double precision)* current_setting('checkpoint_segments')::double precision) + 1) max_wal \
			FROM \
			( \
				SELECT  (pg_stat_file('pg_xlog/' || foo.files )).* FROM \
					( SELECT pg_ls_dir('pg_xlog') files  ) foo \
				WHERE  (pg_stat_file('pg_xlog/' || foo.files )).isdir IS FALSE \
			) bar \
			WHERE bar.size >= 16777216" 

                results = sql.getSQLResult ( {'host': param['host'][0] , 'port' : param['port'][0], 'dbname': dbname, 'user' : param['user'] ,'password' : param['password'] } ,query )

		if results[0] == None : 
			return '2' + ' ' + item_name + ' ' + '-' + ' ' + results[1]


		rows = results[1]
		if len(rows) > 0 :
			warning = fac.getNumberPercentLimits( param.get('warning'), rows[0][1])
			critical = fac.getNumberPercentLimits( param.get('critical'), rows[0][1])
                	for row in rows :
                        	if perfdata == '-' :
					perfdata = perf.getPerfStm ('WALS',row[0],warning,str(critical))
                                	output =  '{0:s} WAL file(s) found'.format( str(row[0]) )
	                	status.append( st.getStatus( row[0],int(warning) , int(critical) ) )

                	status.sort( reverse=True )
                	return str(status[0]) + ' ' + item_name + ' ' + str(perfdata) + ' ' + output
		else :
			return '2' + ' ' + item_name + ' ' + '-' + ' ' + 'No WAL file found, at least One WAL file should be present.'

